/* -*- mode:C; tab-width:4; -*- */
/* 
 *  Author: Tomas Frydrych <tf@o-hand.com>
 *
 *  Copyright (c) 2005 - 2006 OpenedHand Ltd - http://o-hand.com
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 */
#include "dates_platform.h"
#include "dates_types.h"
#include "dates_callbacks.h"
#include "dates_navigator.h"

#include <libmokoui/moko-application.h>
#include <libmokoui/moko-paned-window.h>
#include <libmokoui/moko-tool-box.h>
#include <libmokoui/moko-tree-view.h>

#include <libecal/e-cal.h>
#include <libecal/e-cal-time-util.h>
#include <libical/ical.h>

#include <string.h>

#define DATES_ZOOM_DAY 6
#define DATES_ZOOM_WEEK 10
#define DATES_ZOOM_MONTH 11
#define DATES_ZOOM_YEAR 16

#define DATES_FILTER_DAY    0
#define DATES_FILTER_WEEK   1
#define DATES_FILTER_MONTH  2
#define DATES_FILTER_YEAR   3
#define DATES_FILTER_SEARCH 4
#define DATES_FILTER_TODO   5

static void
navigator_changed_cb (GtkTreeSelection* selection, DatesData* d)
{
    g_debug("navigator selection changed");
	
    GtkTreeModel * model;
    GtkTreeIter    iter;
	
    gboolean has_selection = gtk_tree_selection_get_selected (selection,
															  &model, &iter );

	gchar * uid = NULL;
	if (has_selection)
        gtk_tree_model_get (model, &iter, DN_Uid, &uid, -1);

	if (uid)
	{
		if (d->uri_uid)
			g_free (d->uri_uid);
		
		d->uri_uid = uid;

		g_idle_add (dates_select_event_idle_cb, d);
	}
}


static guint
count_tree_rows (DatesData * d)
{
	return gtk_tree_model_iter_n_children (GTK_TREE_MODEL (d->navigator),
                                           NULL);
}

static void
update_status_bar (DatesData * d)
{
#if 0
	char buf[20];
	guint filter_count = count_tree_rows (d);
	/* the status bar is a separate app that we will communicate with over
	 * debus, see http://wiki.openmoko.org/index.php/Footer
	 */
	gtk_statusbar_pop (GTK_STATUSBAR (d->status_bar), 1);

	snprintf (buf, sizeof (buf), "%d", filter_count);

	gtk_statusbar_push (GTK_STATUSBAR (d->status_bar), 1, buf);
#endif
}

static void
populate_navigator (DatesData * d)
{
	gchar         * query  = NULL;
	DNTimeFormat    format = DN_DateAndTime;
	
	if (d->active_filter <= DATES_FILTER_YEAR)
	{
		gchar               * start_str, * end_str;
		struct icaltimetype   start;
		struct icaltimetype   end;
		icaltimezone        * zone = NULL;
		
		dates_view_get_visible_span (d->view, &start, &end);
	
		start_str =
			isodate_from_time_t(icaltime_as_timet_with_zone (start,zone));
		end_str   =
			isodate_from_time_t(icaltime_as_timet_with_zone (end,zone));

		query = g_strdup_printf ("(occur-in-time-range? "
								 "(make-time \"%s\") (make-time \"%s\"))",
								 start_str, end_str);

		if (d->active_filter == DATES_FILTER_DAY)
			format = DN_TimeOnly;
	}
	else if (d->active_filter == DATES_FILTER_SEARCH)
	{
		query = g_strdup ("#t");
	}
	else /* TODO */
	{
		g_debug ("quering for TODO");
		query = g_strdup_printf ("(is-vtype? %d)", (int)E_CAL_COMPONENT_TODO);
	}

	dates_navigator_model_populate (DATES_NAVIGATOR_MODEL (d->navigator),
									d->dcal, query, format);
	
	g_free (query);
}

static gboolean
dates_is_row_visible_cb (GtkTreeModel * model, GtkTreeIter * iter,
						 DatesData * d)
{
	if (d->active_filter != DATES_FILTER_SEARCH)
		return TRUE;
	
	gboolean retval = FALSE;
	
	GtkEntry * entry;
	entry = moko_tool_box_get_entry (MOKO_TOOL_BOX (d->current_toolbar));

	if (!entry)
		return TRUE;
	
	gchar * what = NULL;
	gchar * what_lc = NULL;
	const gchar * etext = gtk_entry_get_text (entry);
	gchar * etext_lc = NULL;
	
	if (!etext || !*etext)
		return TRUE;

	gtk_tree_model_get (model, iter, DN_Name, &what, -1);

	if (!what)
		return FALSE;

	etext_lc = g_utf8_casefold (etext, -1);
	what_lc  = g_utf8_casefold (what, -1);
	
	if (what_lc && etext_lc && strstr (what_lc, etext_lc))
		retval = TRUE;
	
	g_free (what);
	g_free (what_lc);
	g_free (etext_lc);
	
	return retval;
}

static void        
row_activated_cb (GtkTreeView       *tree_view,
                  GtkTreePath       *path,
                  GtkTreeViewColumn *column,
                  gpointer           user_data)
{
  DatesData *d = user_data;
  
  /* Call the event-clicked callback directly to save duplication */
  dates_open_cb (NULL, d);
}

static GtkWidget *
create_navigation_area( DatesData* d )
{
  d->navigator = GTK_TREE_MODEL (dates_navigator_model_new ());
	
	gtk_tree_model_filter_set_visible_func (
		GTK_TREE_MODEL_FILTER (d->navigator),
		(GtkTreeModelFilterVisibleFunc) dates_is_row_visible_cb,
		d, NULL);
	
    MokoTreeView * treeview =
		MOKO_TREE_VIEW(moko_tree_view_new_with_model(GTK_TREE_MODEL(d->navigator)));

    GtkCellRenderer* ren = gtk_cell_renderer_text_new ();
    GtkTreeViewColumn* col =
		gtk_tree_view_column_new_with_attributes(_("What"), ren, "text",
												 DN_Name, NULL);
	moko_tree_view_append_column (treeview, col);
	
	ren = gtk_cell_renderer_text_new ();
    col = gtk_tree_view_column_new_with_attributes(_("When"), ren, "text",
												   DN_Time, NULL);
	moko_tree_view_append_column (treeview, col);

    GtkTreeSelection* selection =
		gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));

    g_signal_connect( G_OBJECT(selection), "changed",
					  G_CALLBACK(navigator_changed_cb), d);

    g_signal_connect (treeview, "row-activated", G_CALLBACK (row_activated_cb), d);

	gtk_widget_show (GTK_WIDGET (treeview));
	return GTK_WIDGET (treeview);
}

static gboolean
moko_filter_changed (GtkWidget* widget, gchar* text, DatesData * d)
{
	guint old_zoom = d->zoom;
	
	if (!strcmp (text, _("Day")))
	{
		d->zoom = DATES_ZOOM_DAY; /* 1 day 24 hours */
		d->active_filter = DATES_FILTER_DAY;
	}
	else if (!strcmp (text, _("Week")))
	{
		d->zoom = DATES_ZOOM_WEEK; /* 7 days 24 hours */
		d->active_filter = DATES_FILTER_WEEK;
	}
	else if (!strcmp (text, _("Month")))
	{
		d->zoom = DATES_ZOOM_MONTH; /* 1 month 7 days 24 hours */
		d->active_filter = DATES_FILTER_MONTH;
	}
	else if (!strcmp (text, _("Year")))
	{
		d->zoom = DATES_ZOOM_YEAR; /* 12 months 7 days 24 hours */
		d->active_filter = DATES_FILTER_YEAR;
	}
	else if (!strcmp (text, _("To Do")))
	{
		d->active_filter = DATES_FILTER_TODO;
	}
	else if (!strcmp (text, _("Search")))
	{
		d->active_filter = DATES_FILTER_SEARCH;
		
		GtkEntry * entry =
			moko_tool_box_get_entry (MOKO_TOOL_BOX (d->current_toolbar));

		GtkWidget * page = gtk_widget_get_parent (GTK_WIDGET (entry));
		
		gint cur_page =
			gtk_notebook_get_current_page (GTK_NOTEBOOK (d->current_toolbar));
		
		gint my_page =
			gtk_notebook_page_num (GTK_NOTEBOOK (d->current_toolbar),
								   page);

		if (cur_page != my_page)
			gtk_notebook_set_current_page (GTK_NOTEBOOK (d->current_toolbar),
										   my_page);

		gtk_widget_grab_focus(GTK_WIDGET (entry));
	}

	if (old_zoom != d->zoom)
	{
		dates_zoom_change (d->zoom, d->view);
	}
	
	populate_navigator (d);
							  
	return FALSE;
}

static GtkWidget *
create_app_menu (DatesData *d)
{
    GtkWidget * appmenu = gtk_menu_new();
	GtkWidget * menuitem;

	menuitem = gtk_menu_item_new_with_label(_("Import (TODO)"));
	gtk_menu_shell_append(GTK_MENU_SHELL (appmenu), menuitem);

	menuitem = gtk_menu_item_new_with_label(_("Export (TODO)"));
	gtk_menu_shell_append(GTK_MENU_SHELL (appmenu), menuitem);

	menuitem = gtk_menu_item_new_with_label(_("Help"));
	gtk_menu_shell_append(GTK_MENU_SHELL (appmenu), menuitem);
	g_signal_connect (G_OBJECT (menuitem), "activate",
					  G_CALLBACK (dates_about_cb), d);
	
	gtk_widget_show (appmenu);
	return appmenu;
}

static void
entry_changed_cb (GtkEntry *entry, gpointer data)
{
	DatesData * d = data;

	if (d->active_filter != DATES_FILTER_SEARCH)
	{
		d->active_filter = DATES_FILTER_SEARCH;
		
		MokoMenuBox * menubox =
			moko_paned_window_get_menubox(MOKO_PANED_WINDOW(d->main_window));

		moko_menu_box_set_active_filter(menubox, _("Search"));
	}
		
	gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (d->navigator));
	update_status_bar (d);
}

void static
connect_search_box (DatesData * d, GtkWidget * toolbar)
{
	GtkEntry * entry = moko_tool_box_get_entry (MOKO_TOOL_BOX (toolbar));

	g_signal_connect ((gpointer) entry, "changed",
					  G_CALLBACK (entry_changed_cb), d);
}

static GtkWidget *
create_details_toolbar (DatesData *  d)
{
	GtkWidget * toolbar = moko_tool_box_new_with_search();
	GtkWidget * button;
    gtk_widget_grab_focus(toolbar);
	gtk_widget_show (toolbar);

	d->TBDelete =
		GTK_WIDGET (moko_tool_box_add_action_button (MOKO_TOOL_BOX (toolbar)));
	gtk_button_set_label (GTK_BUTTON (d->TBDelete), _("Delete"));
	gtk_widget_show (d->TBDelete);

	g_signal_connect (G_OBJECT (d->TBDelete), "clicked",
					  G_CALLBACK (dates_delete_cb), d);

	button =
		GTK_WIDGET (moko_tool_box_add_action_button (MOKO_TOOL_BOX (toolbar)));
	
	gtk_button_set_label (GTK_BUTTON (button), _("Cancel"));
	gtk_widget_show (button);
	
	g_signal_connect (G_OBJECT (button), "clicked",
					  G_CALLBACK (dates_details_cancel_cb), d);

	button =
		GTK_WIDGET (moko_tool_box_add_action_button (MOKO_TOOL_BOX (toolbar)));
	
	gtk_button_set_label (GTK_BUTTON (button), _("OK"));
	gtk_widget_show (button);
	
	g_signal_connect (G_OBJECT (button), "clicked",
					  G_CALLBACK (dates_details_ok_cb), d);

	connect_search_box (d, toolbar);
	
	gtk_widget_show_all (toolbar);
	return toolbar;
}


static void
set_active_filter (DatesData *d)
{
	MokoMenuBox * menubox =
		moko_paned_window_get_menubox(MOKO_PANED_WINDOW(d->main_window));

	if (d->zoom == DATES_ZOOM_DAY)
		moko_menu_box_set_active_filter(menubox, _("Day"));
	else if (d->zoom == DATES_ZOOM_WEEK)
		moko_menu_box_set_active_filter(menubox, _("Week"));
	else if (d->zoom == DATES_ZOOM_MONTH)
		moko_menu_box_set_active_filter(menubox, _("Month"));
	else if (d->zoom == DATES_ZOOM_YEAR)
		moko_menu_box_set_active_filter(menubox, _("Year"));
	else
		g_warning ("%s: zoom %d", __FUNCTION__, d->zoom);
}

#define DATES_PLATFORM_create_main_window
static GtkWidget *
create_main_window (DatesData * d, GtkWidget * toolbar,
					GtkWidget * menu, GtkAccelGroup * accel_group)
{
	GtkWidget    * main_vbox;
	GtkWidget    * appmenu;
	MokoMenuBox  * menubox;
	GtkWidget    * navigator;
    
    /* Temporary hack */
	GtkSettings *settings = gtk_settings_get_default ();
	g_object_set (settings,
			"gtk-theme-name", "openmoko-standard",
			NULL);

    d->main_window = moko_paned_window_new();

	main_vbox = gtk_vbox_new (FALSE, 0);
	d->calendar_vbox = main_vbox;
	gtk_widget_ref (main_vbox);
	
	gtk_widget_show (main_vbox);
	moko_paned_window_set_lower_pane(MOKO_PANED_WINDOW (d->main_window),
									 main_vbox);

	navigator = create_navigation_area (d);
    moko_paned_window_set_upper_pane (MOKO_PANED_WINDOW (d->main_window),
		  GTK_WIDGET(moko_tree_view_put_into_scrolled_window(MOKO_TREE_VIEW(navigator))));
	
	gtk_widget_show (menu);
    moko_paned_window_set_filter_menu(MOKO_PANED_WINDOW (d->main_window),
									  GTK_MENU(menu));

    menubox = moko_paned_window_get_menubox(MOKO_PANED_WINDOW(d->main_window));
	
	appmenu = create_app_menu (d);
    moko_paned_window_set_application_menu(MOKO_PANED_WINDOW (d->main_window),
										   GTK_MENU (appmenu));

	d->calendar_toolbar = d->current_toolbar = toolbar;
	gtk_widget_ref (toolbar);
	
	moko_paned_window_add_toolbox(MOKO_PANED_WINDOW (d->main_window),
								  MOKO_TOOL_BOX (toolbar));

	d->details_toolbar = create_details_toolbar (d);
	gtk_widget_ref (d->details_toolbar);
	
	d->header_eventbox = gtk_event_box_new ();
	gtk_box_pack_start (GTK_BOX (main_vbox), d->header_eventbox,
						FALSE, FALSE, 0);

	d->header_label = gtk_label_new (_("<big><b>Dates</b></big>"));
	gtk_widget_show (d->header_label);
	gtk_container_add (GTK_CONTAINER (d->header_eventbox), d->header_label);
	gtk_label_set_use_markup (GTK_LABEL (d->header_label), TRUE);

	gtk_widget_show (GTK_WIDGET (d->view));
	gtk_box_pack_start (GTK_BOX (main_vbox), GTK_WIDGET(d->view),
						TRUE, TRUE, 0);
	
	GTK_WIDGET_SET_FLAGS (d->view, GTK_CAN_FOCUS);
	GTK_WIDGET_UNSET_FLAGS (d->view, GTK_CAN_DEFAULT);

	gtk_widget_grab_focus (GTK_WIDGET (d->view));

	/* Set nice colours for full-screen date header */
	gtk_widget_set_state (d->header_eventbox, GTK_STATE_SELECTED);
	gtk_widget_set_state (d->header_label, GTK_STATE_SELECTED);

	g_signal_connect (G_OBJECT (d->main_window), "window_state_event",
					  G_CALLBACK (dates_window_state_cb), d);
	g_signal_connect (G_OBJECT (d->main_window), "key_press_event",
					  G_CALLBACK (dates_key_press_cb), d);

    g_signal_connect( G_OBJECT(menubox), "filter_changed",
					  G_CALLBACK(moko_filter_changed), d );

	gtk_widget_show_all (d->main_window);
	
	return d->main_window;
}

#define DATES_PLATFORM_create_main_menu
static GtkWidget *
create_main_menu (DatesData * d, GtkAccelGroup ** accel_group)
{
	GtkWidget     * menuitem;

	*accel_group = NULL;
	d->main_menu = gtk_menu_new ();

	menuitem = gtk_menu_item_new_with_label (_("Day"));
	gtk_widget_show (menuitem);
	gtk_container_add (GTK_CONTAINER (d->main_menu), menuitem);

	menuitem = gtk_menu_item_new_with_label (_("Week"));
	gtk_widget_show (menuitem);
	gtk_container_add (GTK_CONTAINER (d->main_menu), menuitem);

	menuitem = gtk_menu_item_new_with_label (_("Month"));
	gtk_widget_show (menuitem);
	gtk_container_add (GTK_CONTAINER (d->main_menu), menuitem);

	menuitem = gtk_menu_item_new_with_label (_("Year"));
	gtk_widget_show (menuitem);
	gtk_container_add (GTK_CONTAINER (d->main_menu), menuitem);

	menuitem = gtk_menu_item_new_with_label (_("To Do"));
	gtk_widget_show (menuitem);
	gtk_container_add (GTK_CONTAINER (d->main_menu), menuitem);

	menuitem = gtk_menu_item_new_with_label (_("Search"));
	gtk_widget_show (menuitem);
	gtk_container_add (GTK_CONTAINER (d->main_menu), menuitem);
	
	return d->main_menu;
}

static void
back_cb (GtkButton *button, DatesData *data)
{
	dates_back_cb (button, data);
	populate_navigator (data);
}

static void
forward_cb (GtkButton *button, DatesData *data)
{
	dates_forward_cb (button, data);
	populate_navigator (data);
}

static void
today_cb (GtkButton *button, DatesData *data)
{
	dates_today_cb (button, data);
	populate_navigator (data);
}

#define DATES_PLATFORM_create_toolbar
static GtkWidget *
create_toolbar (DatesData *  d)
{
	GtkWidget * toolbar = moko_tool_box_new_with_search();
	
    gtk_widget_grab_focus(toolbar);
	gtk_widget_show (toolbar);

	d->TBBack =
		GTK_WIDGET (moko_tool_box_add_action_button (MOKO_TOOL_BOX (toolbar)));
	
	gtk_button_set_label (GTK_BUTTON (d->TBBack), _("Back"));
	gtk_widget_show (d->TBBack);
	gtk_button_set_focus_on_click (GTK_BUTTON (d->TBBack), FALSE);

	d->TBToday =
		GTK_WIDGET (moko_tool_box_add_action_button (MOKO_TOOL_BOX (toolbar)));
	
	gtk_button_set_label (GTK_BUTTON (d->TBToday), _("Today"));
	gtk_widget_show (d->TBToday);
	gtk_button_set_focus_on_click (GTK_BUTTON (d->TBToday), FALSE);

	d->TBForward =
		GTK_WIDGET (moko_tool_box_add_action_button (MOKO_TOOL_BOX (toolbar)));
	
	gtk_button_set_label (GTK_BUTTON (d->TBForward), _("Forward"));
	gtk_widget_show (d->TBForward);
	gtk_button_set_focus_on_click (GTK_BUTTON (d->TBForward), FALSE);
	
	g_signal_connect (G_OBJECT (d->TBBack), "clicked",
					  G_CALLBACK (back_cb), d);
	g_signal_connect (G_OBJECT (d->TBToday), "clicked",
					  G_CALLBACK (today_cb), d);
	g_signal_connect (G_OBJECT (d->TBForward), "clicked",
					  G_CALLBACK (forward_cb), d);

	connect_search_box (d, toolbar);
	
	gtk_widget_show_all (toolbar);
	return toolbar;
}

#if 0
static GtkWidget *
create_todo_dialog (DatesData * d)
{
	GtkWidget *vbox;
	GtkWidget *notebook;
	GtkWidget *summary_table;
	GtkWidget *details_calendar_label;
	GtkWidget *label;
	GtkWidget *alignment;
	GtkWidget *hbox;
	GtkWidget *details_end_button;
	GtkWidget *details_table;
	GtkWidget *scrolledwindow1;
	GtkWidget *togglebutton;
	GtkWidget *image;
	GtkWidget *todo_summary_entry;
	GtkWidget *todo_calendar_combobox;
	
	vbox = gtk_vbox_new (FALSE, 0);
	d->details_dialog = vbox;
	gtk_widget_ref (vbox);
	
	gtk_widget_show (vbox);

	notebook = gtk_notebook_new ();
	gtk_widget_show (notebook);
	gtk_box_pack_start (GTK_BOX (vbox), notebook, TRUE, TRUE, 0);

	summary_table = gtk_table_new (3, 2, FALSE);
	gtk_widget_show (summary_table);
	gtk_container_add (GTK_CONTAINER (notebook), summary_table);
	gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (notebook), summary_table,
										TRUE, TRUE, GTK_PACK_START);
	gtk_container_set_border_width (GTK_CONTAINER (summary_table), 12);
	gtk_table_set_row_spacings (GTK_TABLE (summary_table), 6);
	gtk_table_set_col_spacings (GTK_TABLE (summary_table), 6);

	details_calendar_label = gtk_label_new (_("Calendar:"));
	gtk_widget_show (details_calendar_label);
	gtk_table_attach (GTK_TABLE (summary_table), details_calendar_label,
					  0, 1, 0, 1,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);
	
	gtk_misc_set_alignment (GTK_MISC (details_calendar_label), 0, 0.5);

	todo_calendar_combobox = gtk_combo_box_new_text ();
	gtk_widget_show (todo_calendar_combobox);
	gtk_table_attach (GTK_TABLE (summary_table), todo_calendar_combobox,
					  1, 2, 0, 1,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_combo_box_set_focus_on_click (GTK_COMBO_BOX (todo_calendar_combobox),
									  FALSE);

	label = gtk_label_new (_("Summary:"));
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (summary_table), label, 0, 1, 1, 2,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

	todo_summary_entry = gtk_entry_new ();
	gtk_widget_show (todo_summary_entry);
	gtk_table_attach (GTK_TABLE (summary_table), todo_summary_entry,
					  1, 2, 1, 2,
					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);
	gtk_entry_set_activates_default (GTK_ENTRY (todo_summary_entry),
									 TRUE);

	label = gtk_label_new (_("Time:"));
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (summary_table), label, 0, 1, 2, 3,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

	alignment = gtk_alignment_new (0.5, 0.5, 0, 1);
	gtk_widget_show (alignment);
	gtk_table_attach (GTK_TABLE (summary_table), alignment, 1, 2, 2, 3,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (GTK_FILL), 0, 0);


	details_end_button = gtk_button_new ();
	gtk_widget_show (details_end_button);
	gtk_container_add (GTK_CONTAINER (alignment), hbox);

	label = gtk_label_new (_("Summary"));
	gtk_widget_show (label);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook),
								gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 0),
								label);

	details_table = gtk_table_new (1, 2, FALSE);
	gtk_widget_show (details_table);
	gtk_container_add (GTK_CONTAINER (notebook), details_table);
	gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (notebook), details_table,
										TRUE, TRUE, GTK_PACK_START);
	gtk_container_set_border_width (GTK_CONTAINER (details_table), 12);
	gtk_table_set_row_spacings (GTK_TABLE (details_table), 6);
	gtk_table_set_col_spacings (GTK_TABLE (details_table), 6);

	label = gtk_label_new (_("Details:"));
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (details_table), label, 0, 1, 0, 1,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);

	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow1);
	gtk_table_attach (GTK_TABLE (details_table), scrolledwindow1, 1, 2, 0, 1,
					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					  (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
					  0, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1),
									GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(scrolledwindow1),
										 GTK_SHADOW_IN);

	d->details_textview = gtk_text_view_new ();
	gtk_widget_show (d->details_textview);
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), d->details_textview);
	gtk_text_view_set_accepts_tab (GTK_TEXT_VIEW (d->details_textview), FALSE);
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (d->details_textview),
								 GTK_WRAP_WORD);

	label = gtk_label_new (_("Details"));
	gtk_widget_show (label);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook),
								gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 1),
								label);

	gtk_widget_grab_focus (todo_summary_entry);
  
	g_signal_connect (G_OBJECT (d->details_dialog), "delete_event",
					  G_CALLBACK (dates_todo_close_cb), d);
	g_signal_connect (G_OBJECT (todo_calendar_combobox), "changed",
					  G_CALLBACK (dates_calendar_combo_changed_cb), d);

    /* FIXME */
	g_signal_connect (G_OBJECT (details_end_button), "clicked",
					  G_CALLBACK (dates_details_time_end_cb), d);

	return d->details_dialog;
}
#endif

#define DATES_PLATFORM_create_details_dialog
static GtkWidget *
create_details_dialog (DatesData * d)
{
	GtkWidget *vbox;
	GtkWidget *notebook;
	GtkWidget *summary_table;
	GtkWidget *details_calendar_label;
	GtkWidget *label;
	GtkWidget *alignment;
	GtkWidget *hbox;
	GtkWidget *details_start_button;
	GtkWidget *details_end_button;
	GtkWidget *scrolledwindow1;
	GtkWidget *recur_table;
	GtkWidget *recur_repeats_label;
	GtkWidget *recur_combobox;
	GtkWidget *recur_on_label;
	GtkWidget *recur_until_label;
	GtkWidget *recur_until_button;
	GtkWidget *repeats_time_label;
	GtkWidget *recur_week_hbox;
	GtkWidget *togglebutton;
	GtkWidget *recur_except_button;
	GtkWidget *image;
	
	vbox = gtk_vbox_new (FALSE, 0);
	d->details_dialog = vbox;
	gtk_widget_ref (vbox);
	
	gtk_widget_show (vbox);

	notebook = gtk_notebook_new ();
	gtk_widget_show (notebook);
	gtk_box_pack_start (GTK_BOX (vbox), notebook, TRUE, TRUE, 0);

	summary_table = gtk_table_new (3, 2, FALSE);
	gtk_widget_show (summary_table);
	gtk_container_add (GTK_CONTAINER (notebook), summary_table);
	gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (notebook), summary_table,
										TRUE, TRUE, GTK_PACK_START);
	gtk_container_set_border_width (GTK_CONTAINER (summary_table), 12);
	gtk_table_set_row_spacings (GTK_TABLE (summary_table), 6);
	gtk_table_set_col_spacings (GTK_TABLE (summary_table), 6);

	details_calendar_label = gtk_label_new (_("Calendar:"));
	gtk_widget_show (details_calendar_label);
	gtk_table_attach (GTK_TABLE (summary_table), details_calendar_label,
					  0, 1, 0, 1,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (details_calendar_label), 0, 0.5);

	d->details_calendar_combobox = gtk_combo_box_new_text ();
	gtk_widget_show (d->details_calendar_combobox);
	gtk_table_attach (GTK_TABLE (summary_table), d->details_calendar_combobox,
					  1, 2, 0, 1,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_combo_box_set_focus_on_click (GTK_COMBO_BOX (d->details_calendar_combobox),
									  FALSE);
	
	label = gtk_label_new (_("Summary:"));
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (summary_table), label, 0, 1, 1, 2,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

	d->details_summary_entry = gtk_entry_new ();
	gtk_widget_show (d->details_summary_entry);
	gtk_table_attach (GTK_TABLE (summary_table), d->details_summary_entry,
					  1, 2, 1, 2,
					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);
	gtk_entry_set_activates_default (GTK_ENTRY (d->details_summary_entry),
									 TRUE);

	label = gtk_label_new (_("Time:"));
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (summary_table), label, 0, 1, 2, 3,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

	alignment = gtk_alignment_new (0.5, 0.5, 0, 1);
	gtk_widget_show (alignment);
	gtk_table_attach (GTK_TABLE (summary_table), alignment, 1, 2, 2, 3,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (GTK_FILL), 0, 0);

	hbox = gtk_hbox_new (FALSE, 6);
	gtk_widget_show (hbox);
	gtk_container_add (GTK_CONTAINER (alignment), hbox);

	details_start_button = gtk_button_new ();
	gtk_widget_show (details_start_button);
	gtk_box_pack_start (GTK_BOX (hbox), details_start_button, TRUE, TRUE, 0);

	d->details_start_label = gtk_label_new (_("Start"));
	gtk_widget_show (d->details_start_label);
	gtk_container_add (GTK_CONTAINER (details_start_button),
					   d->details_start_label);
	gtk_label_set_use_markup (GTK_LABEL (d->details_start_label), TRUE);
	gtk_label_set_justify (GTK_LABEL (d->details_start_label),
						   GTK_JUSTIFY_CENTER);
	gtk_misc_set_padding (GTK_MISC (d->details_start_label), 3, 3);

	label = gtk_label_new (_("to"));
	gtk_widget_show (label);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

	details_end_button = gtk_button_new ();
	gtk_widget_show (details_end_button);
	gtk_box_pack_start (GTK_BOX (hbox), details_end_button, TRUE, TRUE, 0);

	d->details_end_label = gtk_label_new (_("End"));
	gtk_widget_show (d->details_end_label);
	gtk_container_add (GTK_CONTAINER (details_end_button),
					   d->details_end_label);
	gtk_label_set_justify (GTK_LABEL (d->details_end_label),
						   GTK_JUSTIFY_CENTER);
	gtk_misc_set_padding (GTK_MISC (d->details_end_label), 3, 3);

	label = gtk_label_new (_("Summary"));
	gtk_widget_show (label);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook),
								gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 0),
								label);

	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_container_add (GTK_CONTAINER (notebook), scrolledwindow1);
	gtk_container_set_border_width (GTK_CONTAINER (scrolledwindow1), 12);
	gtk_widget_show (scrolledwindow1);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1),
									GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(scrolledwindow1),
										 GTK_SHADOW_IN);

	d->details_textview = gtk_text_view_new ();
	gtk_widget_show (d->details_textview);
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), d->details_textview);
	gtk_text_view_set_accepts_tab (GTK_TEXT_VIEW (d->details_textview), FALSE);
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (d->details_textview),
								 GTK_WRAP_WORD);

	label = gtk_label_new (_("Details"));
	gtk_widget_show (label);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook),
								gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 1),
								label);
	gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (notebook), scrolledwindow1,
										TRUE, TRUE, GTK_PACK_START);

	recur_table = gtk_table_new (4, 2, FALSE);
	gtk_widget_show (recur_table);
	gtk_container_add (GTK_CONTAINER (notebook), recur_table);
	gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (notebook), recur_table,
										TRUE, TRUE, GTK_PACK_START);
	gtk_container_set_border_width (GTK_CONTAINER (recur_table), 12);
	gtk_table_set_row_spacings (GTK_TABLE (recur_table), 6);
	gtk_table_set_col_spacings (GTK_TABLE (recur_table), 6);

	recur_repeats_label = gtk_label_new (_("Repeats:"));
	gtk_widget_show (recur_repeats_label);
	gtk_table_attach (GTK_TABLE (recur_table), recur_repeats_label, 0, 1, 0, 1,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);
	gtk_label_set_justify (GTK_LABEL (recur_repeats_label), GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (recur_repeats_label), 0, 0.5);

	recur_combobox = gtk_combo_box_new_text ();
	gtk_widget_show (recur_combobox);
	gtk_table_attach (GTK_TABLE (recur_table), recur_combobox, 1, 2, 0, 1,
					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					  (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_widget_set_sensitive (recur_combobox, FALSE);
	gtk_combo_box_append_text (GTK_COMBO_BOX (recur_combobox), _("None"));
	gtk_combo_box_append_text (GTK_COMBO_BOX (recur_combobox), _("Every Day"));
	gtk_combo_box_append_text (GTK_COMBO_BOX (recur_combobox),
							   _("Every Week"));
	gtk_combo_box_append_text (GTK_COMBO_BOX (recur_combobox),
							   _("Every Fortnight"));
	gtk_combo_box_append_text (GTK_COMBO_BOX (recur_combobox),
							   _("Every Month"));
	gtk_combo_box_append_text (GTK_COMBO_BOX (recur_combobox),
							   _("Every Year"));
	gtk_combo_box_append_text (GTK_COMBO_BOX (recur_combobox),
							   _("Custom"));

	recur_on_label = gtk_label_new (_("On:"));
	gtk_table_attach (GTK_TABLE (recur_table), recur_on_label, 0, 1, 1, 2,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (recur_on_label), 0, 0.5);

	recur_until_label = gtk_label_new (_("Until:"));
	gtk_table_attach (GTK_TABLE (recur_table), recur_until_label, 0, 1, 2, 3,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);
	gtk_label_set_justify (GTK_LABEL (recur_until_label), GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (recur_until_label), 0, 0.5);

	recur_until_button = gtk_button_new ();
	gtk_table_attach (GTK_TABLE (recur_table), recur_until_button, 1, 2, 2, 3,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);

	repeats_time_label = gtk_label_new (_("Forever"));
	gtk_widget_show (repeats_time_label);
	gtk_container_add (GTK_CONTAINER (recur_until_button), repeats_time_label);

	recur_week_hbox = gtk_hbox_new (TRUE, 0);
	gtk_table_attach (GTK_TABLE (recur_table), recur_week_hbox, 1, 2, 1, 2,
					  (GtkAttachOptions) (0),
					  (GtkAttachOptions) (GTK_FILL), 0, 0);

	togglebutton = gtk_toggle_button_new_with_mnemonic (_("S"));
	gtk_widget_show (togglebutton);
	gtk_box_pack_start (GTK_BOX (recur_week_hbox), togglebutton,
						TRUE, TRUE, 0);

	togglebutton = gtk_toggle_button_new_with_mnemonic (_("M"));
	gtk_widget_show (togglebutton);
	gtk_box_pack_start (GTK_BOX (recur_week_hbox), togglebutton,
						TRUE, TRUE, 0);

	togglebutton = gtk_toggle_button_new_with_mnemonic (_("T"));
	gtk_widget_show (togglebutton);
	gtk_box_pack_start (GTK_BOX (recur_week_hbox), togglebutton,
						TRUE, TRUE, 0);

	togglebutton = gtk_toggle_button_new_with_mnemonic (_("W"));
	gtk_widget_show (togglebutton);
	gtk_box_pack_start (GTK_BOX (recur_week_hbox), togglebutton,
						TRUE, TRUE, 0);

	togglebutton = gtk_toggle_button_new_with_mnemonic (_("T"));
	gtk_widget_show (togglebutton);
	gtk_box_pack_start (GTK_BOX (recur_week_hbox), togglebutton,
						TRUE, TRUE, 0);

	togglebutton = gtk_toggle_button_new_with_mnemonic (_("F"));
	gtk_widget_show (togglebutton);
	gtk_box_pack_start (GTK_BOX (recur_week_hbox), togglebutton,
						TRUE, TRUE, 0);

	togglebutton = gtk_toggle_button_new_with_mnemonic (_("S"));
	gtk_widget_show (togglebutton);
	gtk_box_pack_start (GTK_BOX (recur_week_hbox), togglebutton,
						TRUE, TRUE, 0);

	alignment = gtk_alignment_new (1, 1, 0, 0);
	gtk_widget_show (alignment);
	gtk_table_attach (GTK_TABLE (recur_table), alignment, 0, 2, 3, 4,
					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

	recur_except_button = gtk_button_new ();
	gtk_widget_show (recur_except_button);
	gtk_container_add (GTK_CONTAINER (alignment), recur_except_button);
	gtk_widget_set_sensitive (recur_except_button, FALSE);

	alignment = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment);
	gtk_container_add (GTK_CONTAINER (recur_except_button), alignment);

	hbox = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox);
	gtk_container_add (GTK_CONTAINER (alignment), hbox);

	image = gtk_image_new_from_stock ("gtk-remove", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show (image);
	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);

	label = gtk_label_new_with_mnemonic (_("_Exceptions"));
	gtk_widget_show (label);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

	label = gtk_label_new (_("Repeats"));
	gtk_widget_show (label);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook),
								gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 2),
								label);

	gtk_widget_grab_focus (d->details_summary_entry);
  
	g_signal_connect (G_OBJECT (d->details_dialog), "delete_event",
					  G_CALLBACK (dates_details_close_cb), d);
	g_signal_connect (G_OBJECT (d->details_calendar_combobox), "changed",
					  G_CALLBACK (dates_calendar_combo_changed_cb), d);
	g_signal_connect (G_OBJECT (details_start_button), "clicked",
					  G_CALLBACK (dates_details_time_start_cb), d);
	g_signal_connect (G_OBJECT (details_end_button), "clicked",
					  G_CALLBACK (dates_details_time_end_cb), d);

	return d->details_dialog;
}

#define DATES_PLATFORM_dates_platform_details_dlg
void
dates_platform_details_dlg (DatesData * d, gboolean show)
{
	if (show)
	{
		GtkWidget * parent = gtk_widget_get_parent (d->calendar_vbox);

		gtk_widget_hide (d->calendar_vbox);
		
		if (parent)
		{
			gtk_container_remove(GTK_CONTAINER(parent), d->calendar_vbox);
			gtk_container_add (GTK_CONTAINER (parent), d->details_dialog);
		}
		else
		{
			moko_paned_window_set_lower_pane(MOKO_PANED_WINDOW(d->main_window),
											 d->details_dialog);
		}

		
		
		if (!(parent = gtk_widget_get_parent (d->details_toolbar)))
		{
			if ((parent = gtk_widget_get_parent (d->calendar_toolbar)))
			{
				gtk_container_remove (GTK_CONTAINER (parent),
									  d->calendar_toolbar);
			}
			/* need to add remove for other toolbars if any are added later */
			
			if (parent)
			{
				moko_paned_window_add_toolbox(MOKO_PANED_WINDOW(d->main_window),
											  MOKO_TOOL_BOX (d->details_toolbar));
				d->current_toolbar = d->details_toolbar;
			}
			else
			{
				g_warning ("Unknown toolbar parented with window (%s:%d)",
						   __FILE__, __LINE__);
			}
		}
		
		gtk_widget_set_size_request (gtk_widget_get_parent (d->details_dialog),
									 -1, -1);
									 
		gtk_widget_show (d->details_dialog);
	}
	else
	{
		GtkWidget * parent = gtk_widget_get_parent (d->details_dialog);

		gtk_widget_hide (d->details_dialog);

		if (parent)
		{
			gtk_container_remove(GTK_CONTAINER(parent), d->details_dialog);
			gtk_container_add (GTK_CONTAINER (parent), d->calendar_vbox);

			gtk_widget_set_size_request
				(gtk_widget_get_parent (d->calendar_vbox),
				 -1, -1);
			
			gtk_widget_show (d->calendar_vbox);
		}

		if (!(parent = gtk_widget_get_parent (d->calendar_toolbar)))
		{
			if ((parent = gtk_widget_get_parent (d->details_toolbar)))
			{
				gtk_container_remove (GTK_CONTAINER (parent),
									  d->details_toolbar);
			}
			/* need to add remove for other toolbars if any are added later */
			
			if (parent)
			{
				moko_paned_window_add_toolbox(MOKO_PANED_WINDOW(d->main_window),
											  MOKO_TOOL_BOX (d->calendar_toolbar));
				d->current_toolbar = d->calendar_toolbar;
			}
			else
			{
				g_warning ("Unknown toolbar parented with window (%s:%d)",
						   __FILE__, __LINE__);
			}
		}
		
	}
}

#define DATES_PLATFORM_dates_platform_init
void
dates_platform_init (DatesData * d)
{
    d->app = MOKO_APPLICATION(moko_application_get_instance());
    g_set_application_name( "Calendar" );
}

#define DATES_PLATFORM_dates_platform_cal_open
void dates_platform_cal_open (DatesData * d)
{
	populate_navigator (d);
}

#define DATES_PLATFORM_dates_platform_pre_show
void
dates_platform_pre_show (DatesData *d)
{
	set_active_filter (d);
}

/* The rest we 'inherit' */
#include "dates_gtk.c"
